import 'dart:math';
import 'dart:typed_data';
import 'dart:io';
import 'dart:async';
import 'package:flutter/services.dart';
import 'package:flutter/material.dart';
import 'live_player_config.dart';
import '../live_push_def.dart';

class AlivcLivePlayer {
  static MethodChannel methodChannel =
      MethodChannel(AlivcLivePusherInterfaceKey.LivePlayerMethodName);
  static EventChannel eventChannel =
      EventChannel(AlivcLivePusherInterfaceKey.LivePlayerEventName);

  OnLivePlayerError? onError;
  OnLivePlayerCallback? onPlayStarted;
  OnLivePlayerCallback? onPlayStoped;
  OnLivePlayerCallback? onFirstVideoFrameDrawn;
  OnNetworkQualityChanged? onNetworkQualityChanged;

  /// 创建[AlivcLivePlayer]实例
  AlivcLivePlayer.init() {
    eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  /// 绑定直播连麦播放参数配置
  ///
  /// 将[AlivcLivePlayConfig]同[AlivcLivePlayer]联系起来。
  Future<void> bindPlayCofig() async {
    return AlivcLivePlayer.methodChannel.invokeMethod('bindPlayCofig', _wrap());
  }

  /// 初始化直播连麦播放引擎
  Future<void> initLivePlayer() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('initLivePlayer', _wrap());
  }

  /// 设置播放回调
  Future<void> setLivePlayerDelegate() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('setLivePlayerDelegate', _wrap());
  }

  /// 设置播放View
  Future<void> setPlayView() async {
    return AlivcLivePlayer.methodChannel.invokeMethod('setPlayView', _wrap());
  }

  /// 开始播放音视频流
  Future<void> startPlayWithURL(String url) async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('startPlayWithURL', _wrap(arg: url));
  }

  /// 停止播放音视频流
  Future<void> stopPlay() async {
    return AlivcLivePlayer.methodChannel.invokeMethod('stopPlay', _wrap());
  }

  /// 暂停播放音频流
  Future<void> pauseAudioPlaying() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('pauseAudioPlaying', _wrap());
  }

  /// 恢复播放音频流
  Future<void> resumeAudioPlaying() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('resumeAudioPlaying', _wrap());
  }

  /// 暂停播放视频流
  Future<void> pauseVideoPlaying() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('pauseVideoPlaying', _wrap());
  }

  /// 恢复播放视频流
  Future<void> resumeVideoPlaying() async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('resumeVideoPlaying', _wrap());
  }

  /// 设置播放音量
  ///
  /// [volume] 播放音量，取值范围[0,400] 0：静音，<100：减小音量，>100：放大音量
  Future<void> setPlayoutVolume(int volume) async {
    return AlivcLivePlayer.methodChannel
        .invokeMethod('resumeVideoPlaying', _wrap(arg: volume.toString()));
  }

  /// 播放错误回调
  ///
  /// [onError] param [errorCode] 错误码
  ///
  /// [onError] param [errorMessage] 错误信息
  void setOnError(OnLivePlayerError onError) {
    this.onError = onError;
  }

  /// 开始播放回调
  void setOnPlayStarted(OnLivePlayerCallback onPlayStarted) {
    this.onPlayStarted = onPlayStarted;
  }

  /// 结束播放回调
  void setOnPlayStoped(OnLivePlayerCallback onPlayStoped) {
    this.onPlayStoped = onPlayStoped;
  }

  /// 视频首帧渲染回调
  void setOnFirstVideoFrameDrawn(OnLivePlayerCallback onFirstVideoFrameDrawn) {
    this.onFirstVideoFrameDrawn = onFirstVideoFrameDrawn;
  }

  /// 网络质量变化时发出的消息
  void setOnNetworkQualityChanged(
      OnNetworkQualityChanged onNetworkQualityChanged) {
    this.onNetworkQualityChanged = onNetworkQualityChanged;
  }

  // 统一回调接口
  void _onEvent(dynamic event) {
    String method = event[AlivcLivePlayerEventChannelDef.Method];
    switch (method) {
      case "onError":
        if (this.onError != null) {
          int errorCodeIntV =
              int.parse(event[AlivcLivePlayerEventChannelDef.ErrorCode]);
          String errorMessage =
              event[AlivcLivePlayerEventChannelDef.ErrorMessage];
          this.onError!(
              AlivcLivePlayerErrorData.convertEnumValue(errorCodeIntV),
              errorMessage);
        }
        break;
      case "onPlayStarted":
        if (this.onPlayStarted != null) {
          this.onPlayStarted!();
        }
        break;
      case "onPlayStoped":
        if (this.onPlayStoped != null) {
          this.onPlayStoped!();
        }
        break;
      case "onFirstVideoFrameDrawn":
        if (this.onFirstVideoFrameDrawn != null) {
          this.onFirstVideoFrameDrawn!();
        }
        break;
      case "onNetworkQualityChanged":
        if (this.onNetworkQualityChanged != null) {
          int qualityIntV =
              int.parse(event[AlivcLivePlayerEventChannelDef.Quality]);
          this.onNetworkQualityChanged!(
              AlivcLiveNetworkQuality.values[qualityIntV]);
        }
        break;
    }
  }

  void _onError(dynamic error) {}

  // 接口部分
  _wrap({arg = ''}) {
    var map = {"arg": arg};
    return map;
  }
}
