import 'dart:typed_data';
import 'dart:io';
import 'dart:async';
import 'package:flutter/services.dart';
import 'live_push_def.dart';

class AlivcBase {
  static MethodChannel methodChannel =
      MethodChannel(AlivcLivePusherInterfaceKey.LiveBaseMethodName);
  static EventChannel eventChannel =
      EventChannel(AlivcLivePusherInterfaceKey.LiveBaseEventName);

  OnLicenceCheck? onLicenceCheck;

  /// 创建[AlivcBase]实例
  AlivcBase.init() {
    eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  /// 获取SDK版本号
  ///
  /// [returns] 版本号
  static Future<String> getSdkVersion() async {
    return await methodChannel.invokeMethod('getSdkVersion');
  }

  /// 设置监听回调接口
  Future<void> setObserver() async {
    return methodChannel.invokeMethod('setObserver');
  }

  /// 设置Log级别
  ///
  /// [level] Log级别 默认:[AlivcLivePushLogLevel.error]
  static Future<void> setLogLevel(AlivcLivePushLogLevel level) async {
    return methodChannel.invokeMethod(
        'setLogLevel', _wrap(arg: level.index.toString()));
  }

  /// 启用或禁用控制台日志打印
  ///
  /// [enable] 指定是否启用 true:启用,false:禁用
  static Future<void> setConsoleEnable(bool enable) async {
    return methodChannel.invokeMethod(
        'setConsoleEnable', _wrap(arg: AlivcLiveTool.boolToString(enable)));
  }

  /// 设置Log路径
  ///
  /// [logPath] Log路径
  ///
  /// [maxPartFileSizeInKB] 每个分片最大大小。最终日志总体积是 5*最大分片大小
  static Future<void> setLogPath(
      String logPath, int maxPartFileSizeInKB) async {
    Map params = {
      "logPath": logPath,
      "maxPartFileSizeInKB": maxPartFileSizeInKB.toString(),
    };
    return methodChannel.invokeMethod('setLogPath', _wrap(arg: params));
  }

  /// 注册SDK
  ///
  /// 请在工程的[Info.plist]中增加[AlivcLicenseKey]和[AlivcLicenseFile]字段
  ///
  /// 在[AlivcLicenseKey]中填入您的[LicenseKey]
  ///
  /// 在[AlivcLicenseFile]中填入您的[LicenseFile]路径（相对于mainBundle）；例如您的[LicenseFile]为"license.crt"放到mainBundle下，就填入license.crt
  ///
  /// LicenseKey和LicenseFile的获取请参考文档：https://help.aliyun.com/document_detail/431730.html
  Future<dynamic> registerSDK() async {
    return methodChannel.invokeMethod('registerSDK');
  }

  // 回调部分
  /// SDK Licence 校验接口回调
  ///
  /// [onLicenceCheck] param [result] 校验licence结果[AlivcLiveLicenseCheckResultCodeSuccess]成功，其他表示失败
  ///
  /// [onLicenceCheck] param [reason] 校验licence失败原因
  void setOnLicenceCheck(OnLicenceCheck onLicenceCheck) {
    onLicenceCheck = onLicenceCheck;
  }

  void _onEvent(dynamic event) {
    String method = event[AlivcLivePusherEventChannelDef.Method];
    switch (method) {
      case "onLicenceCheck":
        if (onLicenceCheck != null) {
          int resultInt = int.parse(
              event[AlivcLivePusherEventChannelDef.LicenceCheckResult]);
          AlivcLiveLicenseCheckResultCode result =
              AlivcLiveLicenseCheckResultCode.values[resultInt];
          String reason =
              event[AlivcLivePusherEventChannelDef.LicenceCheckReason];
          onLicenceCheck!(result, reason);
        }
        break;
    }
  }

  void _onError(dynamic error) {}

  // 接口部分
  static _wrap({arg = ''}) {
    var map = {"arg": arg};
    return map;
  }
}
